<!--
Template for displaying messages to users after completing/aborting/failing a transaction
Filled by sixPay/webinterface/pages/ePayments.py:WTransactionUserCallback.getVars
-->
    <table width="60%" align="center" border="0" style="border: 1px solid #777777">
        <tr>
            <td class="groupTitle" colspan="2" style="text-align:center; background:#E5E5E5; color:gray">${ message }</td>
        </tr>
        <tr>
            <td align="center" colspan="2" bgcolor="white" style="padding-bottom:10px">${ message_detail }</td>
        </tr>
    </table>
